# Generated by Django 2.2.6 on 2019-10-08 17:02

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0030_user_color'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='props',
            field=bitfield.models.BitField((('bot', 'Системный пользователь'),), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2 on 2019-04-14 09:20

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0013_auto_20190414_0919'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages',
            name='lastmodified',
            field=models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='messages_state',
            name='lastmodified',
            field=models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='messages_theme',
            name='lastmodified',
            field=models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-07 01:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0011_auto_20190307_0056'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='params',
            options={'verbose_name': 'Сохраненные параметры'},
        ),
    ]



import logging

from django.conf import settings
from django.core.management import BaseCommand

from isc_common.auth.models.user import User
from isc_common.ws.progressStack import ProgressStack

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--mode', type=str)
        parser.add_argument('--qty', type=int)
        parser.add_argument('--id_user', type=int)
        parser.add_argument('--cancel', type=bool)

    def handle(self, *args, **options):
        mode = options.get('mode')
        qty = options.get('qty')
        id_user = options.get('id_user')
        cancel = options.get('cancel', False)

        user = User.objects.get(id=id_user)

        progress = ProgressStack(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{user.username}',
            props=1 if cancel else 0
        )

        if mode == 'open':
            for i in range(qty):
                progress.show(title=f'Тест {i}', label_contents=f'Text {i}', id=i)

            for i in range(qty):
                for j in range(qty):
                    progress.setPercentsDone(percent=j * 10, id=i)

        if mode == 'close':
            for i in range(qty):
                progress.close(id=i)



import logging

from isc_common.fields.code_field import CodeStrictField
from isc_common.models.audit import AuditQuerySet, AuditManager, AuditModel

logger = logging.getLogger(__name__)


class Model_phonesQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        phone = defaults.get('phone')

        if phone != '' and phone is not None:
            phone_item = Phones.objects.getOptional(phone=phone)
            if phone_item is None:
                try:
                    item = super().get(**kwargs)
                    phone_item, _ = Phones.objects.update_or_create(id=item.phone.id, defaults=dict(phone=phone))
                except self.model.DoesNotExist:
                    phone_item = Phones.objects.create(phone=phone)
            defaults = dict(phone=phone_item)
            return super().update_or_create(defaults=defaults, **kwargs)
        else:
            return None, None

class PhonesQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class PhonesManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'description': record.description,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return PhonesQuerySet(self.model, using=self._db)


class Phones(AuditModel):
    phone = CodeStrictField(unique=True)
    objects = PhonesManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Телефоны'



# Generated by Django 2.2.6 on 2019-10-17 11:35

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0003_history_fio'),
    ]

    operations = [
        migrations.AlterField(
            model_name='history',
            name='path',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
    ]



import logging

from django.contrib.postgres.fields import ArrayField
from django.db.models import Model, BooleanField, BigAutoField, DateTimeField, Transform, CharField, TextField, BigIntegerField
from django.utils import timezone
from transliterate import translit
from transliterate.exceptions import LanguageDetectionError

from isc_common import setAttr, delAttr
from isc_common.managers.common_manager import CommonManager, CommonQuerySet

logger = logging.getLogger(__name__)


class Model_withOldId(Model):
    old_id = BigIntegerField(db_index=True, null=True, blank=True, unique=True)

    class Meta:
        abstract = True


class Model_withOldIds(Model):
    old_ids = ArrayField(BigIntegerField(), null=True, blank=True, unique=True)

    class Meta:
        abstract = True


class AuditQuerySet(CommonQuerySet):
    def getOptional(self, *args, **kwargs):
        try:
            return self.get(*args, **kwargs)
        except self.model.DoesNotExist:
            return None

    def _check_ed_izm_qty(self, **kwargs):
        if kwargs.get('ed_izm') is not None and kwargs.get('qty') is None or kwargs.get('ed_izm') is None and kwargs.get('qty') is not None:
            raise Exception(f'Единица измерения не может быть указана без количества')

    def __init__(self, model=None, query=None, using=None, hints=None, alive_only=True):
        self.alive_only = alive_only
        super().__init__(model=model, query=query, using=using, hints=hints, alive_only=alive_only)

    def rearrange_parent(self, json):
        # _criteria = json.get('data').get('criteria')
        # if isinstance(_criteria, list):
        #     for criterion in _criteria:
        #         if criterion.get('fieldName') == 'parent_id':
        #             item_id = criterion.get('value')
        #             if (item_id, int):
        #                 criteria_lst = [criterion for criterion in _criteria if criterion.get('fieldName') == 'parent_id' and criterion.get('value') is not None]
        #                 if len(criteria_lst) > 0:
        #                     setAttr(json.get('data'), 'criteria', criteria_lst)
        return json

    def soft_delete(self):
        res = super().update(deleted_at=timezone.now())
        if res > 0:
            return self
        else:
            return None

    def soft_restore(self):
        res = super().update(deleted_at=None)
        return res

    def hard_delete(self):
        res = super().delete()
        return res

    def alive(self):
        return self.filter(deleted_at=None)

    def dead(self):
        return self.exclude(deleted_at=None)

    def update(self, **kwargs):
        if kwargs.get('lastmodified') is None:
            setAttr(kwargs, 'lastmodified', timezone.now())
        return super().update(**kwargs)


class AuditManager(CommonManager):
    def __init__(self, *args, **kwargs):
        self.alive_only = kwargs.pop('alive_only', True)
        super().__init__(*args, **kwargs)

    def get_queryset(self):
        if self.alive_only:
            return AuditQuerySet(model=self.model, alive_only=self.alive_only).filter(deleted_at=None)
        return AuditQuerySet(model=self.model, alive_only=self.alive_only)

    def hard_delete(self):
        return self.get_queryset().hard_delete()

    def soft_delete(self):
        return self.get_queryset().soft_delete()

    def soft_restore(self):
        return self.get_queryset().soft_restore()

    def getOptional(self, *args, **kwargs):
        try:
            return self.get(*args, **kwargs)
        except self.model.DoesNotExist:
            return None

    def getOr(self, *args, **kwargs):
        alternative = kwargs.get('alternative')
        try:
            delAttr(kwargs, 'alternative')
            return self.get(*args, **kwargs)
        except self.model.DoesNotExist:
            if not callable(alternative):
                raise Exception('alternative must be callable')
            return alternative()


class Manager(CommonManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_queryset(self):
        return AuditQuerySet(model=self.model)


class AuditModel(Model):
    id = BigAutoField(primary_key=True, verbose_name="Идентификатор")
    deleted_at = DateTimeField(verbose_name="Дата мягкого удаления", null=True, blank=True, db_index=True)
    editing = BooleanField(verbose_name="Возможность редактирования", default=True)
    deliting = BooleanField(verbose_name="Возможность удаления", default=True)
    lastmodified = DateTimeField(verbose_name='Последнее обновление', editable=False, db_index=True, default=timezone.now)

    @classmethod
    def uncapitalize(cls, str):
        from isc_common.common.functions import uncapitalize
        return uncapitalize(str)

    @classmethod
    def translit(cls, value):
        if isinstance(value, int):
            return str(value)
        try:
            return cls.uncapitalize(translit(value, reversed=True).replace("'", '').replace(' ', '_').replace('.', '_'))
        except LanguageDetectionError:
            return cls.uncapitalize(value.replace("'", '').replace(' ', '_').replace('.', '_'))

    @classmethod
    def dbl_qutes_str(cls, str):
        from isc_common.common.functions import dbl_qutes_str
        return dbl_qutes_str(str)

    @classmethod
    def qutes_str(cls, str):
        from isc_common.common.functions import qutes_str
        return qutes_str(str)

    @property
    def idHidden(self):
        return not self is None

    @property
    def is_deleted(self):
        return self.deleted_at is not None

    class Meta:
        abstract = True

    def soft_delete(self):
        self.deleted_at = timezone.now()
        self.save()
        return self

    def soft_restore(self):
        self.deleted_at = None
        self.save()
        return self

    objects = AuditManager()
    all_objects = AuditManager(alive_only=False)


class Dbl_spacesValue(Transform):
    lookup_name = 'delete_dbl_spaces'
    function = 'delete_dbl_spaces'


CharField.register_lookup(Dbl_spacesValue)
TextField.register_lookup(Dbl_spacesValue)


class Trim_Value(Transform):
    lookup_name = 'trim'
    function = 'trim'


CharField.register_lookup(Trim_Value)
TextField.register_lookup(Trim_Value)


class Upper_Value(Transform):
    lookup_name = 'upper'
    function = 'upper'


CharField.register_lookup(Upper_Value)
TextField.register_lookup(Upper_Value)



from bitfield import Bit


def IsBitOn(value, bit):
    if isinstance(bit, Bit):
        bit = bit.number

    res = (value & (1 << bit))
    if isinstance(res, int):
        return res
    return (value & (1 << bit))._value != 0


def IsBitOff(value, bit):
    return not IsBitOn(value, bit)


def TurnBitOn(value, bit):
    if isinstance(bit, Bit):
        bit = bit.number

    return value | (1 << bit)


def TurnBitOff(value, bit):
    if isinstance(bit, Bit):
        bit = bit.number

    return value & ~ (1 << bit)


def CopyBit(value, bit):
    if IsBitOn(value, bit):
        return TurnBitOn(value, bit)
    else:
        return TurnBitOff(value, bit)



from django.urls import path

from isc_common.views import last_used_processes

urlpatterns = [

    path('Last_used_processes/Fetch/', last_used_processes.Last_used_processes_Fetch),
    path('Last_used_processes/Add', last_used_processes.Last_used_processes_Add),
    path('Last_used_processes/Update', last_used_processes.Last_used_processes_Update),
    path('Last_used_processes/Remove', last_used_processes.Last_used_processes_Remove),
    path('Last_used_processes/Lookup/', last_used_processes.Last_used_processes_Lookup),
    path('Last_used_processes/Info/', last_used_processes.Last_used_processes_Info),
    path('Last_used_processes/Copy', last_used_processes.Last_used_processes_Copy),

]



import logging

from django.db import transaction

from isc_common import delAttr, setAttr
from isc_common.auth.models.user import User
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_tree_grid_manager import CommonTreeGridManager
from isc_common.models.base_ref import BaseRefHierarcy

logger = logging.getLogger(__name__)


class Messages_theme_Manager(CommonTreeGridManager):
    @classmethod
    def getRecord(cls, record):
        res = {
            "id": record.id,
            "code": record.code,
            "name": record.name,
            # "full_name": record.full_name,
            "description": record.description,
            "parent_id": record.parent_id,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def createFromRequest(self, request):
        from tracker.models.messages_theme_users_access import MessagesThemeUsersAccess

        _request = DSRequest(request=request)
        user_id = _request.user.id
        data = _request.get_data()
        _data = data.copy()
        delAttr(_data, 'id')
        delAttr(_data, 'isFolder')
        delAttr(_data, 'full_name')
        setAttr(_data, 'creator_id', user_id)
        res = super().create(**_data)

        MessagesThemeUsersAccess.objects.get_or_create(theme=res, user_id=user_id)
        return data

    def updateFromRequest(self, request):
        from tracker.models.messages import Messages

        request = DSRequest(request=request)
        data = request.get_data()
        mode = data.get('mode')
        dropRecords = data.get('dropRecords')
        targetRecord = data.get('targetRecord')

        if mode == 'move':
            res = 0
            for dropRecord in dropRecords:
                res += Messages.objects.filter(id=dropRecord).update(theme_id=targetRecord)
            return res

        _list = [value for key, value in data.items()]
        is_list_of_dict = len([a for a in _list if isinstance(a, dict)]) > 0
        if is_list_of_dict:
            with transaction.atomic():
                for _data in _list:
                    if isinstance(_data, dict):
                        delAttr(_data, 'isFolder')
                        delAttr(_data, 'full_name')
                        delAttr(_data, 'children')
                        res = super().filter(id=_data.get('id')).update(**_data)
        else:
            delAttr(data, 'isFolder')
            delAttr(data, 'full_name')
            delAttr(data, 'children')
            res = super().filter(id=data.get('id')).update(**data)
        return res


class Messages_theme(BaseRefHierarcy):
    code = CodeField(unique=True)
    creator = ForeignKeyCascade(User, null=True, blank=True)

    @classmethod
    def auto_from_error(cls):
        return Messages_theme.objects.get_or_create(
            code='auto_from_error',
            defaults=dict(
                name='Автоматически занесенные из сообщений об ошибках.',
                creator=User.admin_user(),
                editing=False,
                deliting=False,
                description='Тема создана автоматически'))[0]

    @classmethod
    def chats_theme(cls):
        return Messages_theme.objects.get_or_create(
            code='chats',
            defaults=dict(
                name='Сообщения чатов',
                creator=User.admin_user(),
                editing=False,
                deliting=False,
                description='Тема создана автоматически'))[0]

    @classmethod
    def group_chats_theme(cls):
        return Messages_theme.objects.get_or_create(
            code='group_chats',
            defaults=dict(
                name='Групповые чаты',
                creator=User.admin_user(),
                editing=False,
                deliting=False,
                parent=Messages_theme.chats_theme(),
                description='Тема создана автоматически'))[0]

    @classmethod
    def tetatet_chats_theme(cls):
        return Messages_theme.objects.get_or_create(
            code='tetatet_chats',
            defaults=dict(
                name='Чаты личной переписки',
                creator=User.admin_user(),
                editing=False,
                deliting=False,
                parent=Messages_theme.chats_theme(),
                description='Тема создана автоматически'))[0]

    @classmethod
    def system_chats_theme(cls):
        return Messages_theme.objects.get_or_create(
            code='system_chats',
            defaults=dict(
                name='Чаты системных сообщений',
                creator=User.admin_user(),
                editing=False,
                deliting=False,
                parent=Messages_theme.chats_theme(),
                description='Тема создана автоматически'))[0]

    def __str__(self):
        return f"id: {self.code}, id: {self.code}, name: {self.name}, description: {self.description}"

    objects = Messages_theme_Manager()

    class Meta:
        verbose_name = 'Темы задач'



import logging

from django.forms import model_to_dict
from django.utils.translation import gettext_lazy as _

from isc_common import delAttr
from isc_common.auth.models.usergroup_permission import Usergroup_permission
from isc_common.auth.models.widgets_trees import Widgets_trees
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager
from isc_common.models.base_ref import BaseRef

logger = logging.getLogger(__name__)


class GroupManager(AuditManager):
    @classmethod
    def getRecord(cls, record):
        res = {
            "id": record.id,
            "code": record.code,
            "name": record.name,
            "description": record.description,
            "parent_id": record.parent_id,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        delAttr(_data, 'dataSource')
        delAttr(_data, 'operationType')
        delAttr(_data, 'textMatchStyle')
        delAttr(_data, 'form')
        delAttr(_data, 'id')

        res = super().create(**_data)
        for widget in Widgets_trees.objects.all():
            Usergroup_permission.objects.get_or_create(usergroup=res, widget=widget)

        res = model_to_dict(res)
        data.update(res)
        return data


class UserGroup(BaseRef):
    code = CodeField(unique=True)
    parent = ForeignKeyProtect('self', null=True, blank=True)

    @property
    def is_admin(self):
        return self.code.lower() == 'administrators' or self.code.lower() == 'admin'

    @property
    def is_develop(self):
        return self.code == 'developers'

    def __str__(self):
        return f"{self.code}"

    objects = GroupManager()

    class Meta:
        verbose_name = _('группа')
        verbose_name_plural = _('группы')



import logging

from isc_common.fields.code_field import ColorField
from isc_common.models.base_ref import BaseRefManager, BaseRef, BaseRefQuerySet

logger = logging.getLogger(__name__)


class Standard_colorsQuerySet(BaseRefQuerySet):
    pass


class Standard_colorsManager(BaseRefManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            'code': record.code,
            'color': record.color,
            'name': record.name,
            'description': record.description,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Standard_colorsQuerySet(self.model, using=self._db)


class Standard_colors(BaseRef):
    color = ColorField(null=True, blank=True)
    objects = Standard_colorsManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Предварительно установленные цвета'



import logging
from datetime import timedelta

from django.db.models import Manager, Model, CharField, DateTimeField, UniqueConstraint, Q, BigAutoField
from django.utils import timezone

from history import utils

logger = logging.getLogger(__name__)


class VisitorManager(Manager):

    def active(self, timeout=None):
        """
        Retrieves only visitors who have been active within the timeout
        period.
        """
        if not timeout:
            timeout = utils.get_timeout()

        now = timezone.now()
        cutoff = now - timedelta(minutes=timeout)

        return self.get_queryset().filter(last_update__gte=cutoff)


class Visitor(Model):
    id = BigAutoField(primary_key=True, verbose_name="Идентификатор")
    session_key = CharField(max_length=40)
    username = CharField(max_length=255, null=True, blank=True)
    ip_address = CharField(max_length=20)
    user_agent = CharField(max_length=255)
    referrer = CharField(max_length=255)
    url = CharField(max_length=255)
    session_start = DateTimeField()
    last_update = DateTimeField()
    objects = VisitorManager()

    def __str__(self):
        return u'{0} at {1} '.format(
            self.username,
            self.ip_address
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.session_start = timezone.now()
        self.last_update = timezone.now()

    @property
    def time_on_site(self):
        """
        Attempts to determine the amount of time a visitor has spent on the
        site based upon their information that's in the database.
        """
        if self.session_start:
            seconds = (self.last_update - self.session_start).seconds

            hours = seconds / 3600
            seconds -= hours * 3600
            minutes = seconds / 60
            seconds -= minutes * 60

            return u'%i:%02i:%02i' % (hours, minutes, seconds)
        else:
            return 'unknown'

    class Meta:
        verbose_name = 'Посетитель'
        ordering = ('-last_update',)
        unique_together = ('username', 'ip_address',)
        constraints = [
            UniqueConstraint(fields=['username', 'ip_address'], name='Visitor_unique_1'),
            UniqueConstraint(fields=['ip_address'], condition=Q(username=None), name='Visitor_unique_2'),
        ]



import logging

from django.contrib.auth.validators import UnicodeUsernameValidator
from django.core.mail import send_mail
from django.db.models import ManyToManyField, DateField
from django.utils.translation import gettext_lazy as _

from isc_common.auth.models.abstract_base_user import AbstractBaseUser
from isc_common.auth.models.usergroup import UserGroup
from isc_common.fields.code_field import CodeStrictField
from isc_common.fields.name_field import NameField
from isc_common.models.audit import AuditModel

logger = logging.getLogger(__name__)

class NonamedUserException(Exception):
    pass

class AbstractUser(AbstractBaseUser, AuditModel):
    username_validator = UnicodeUsernameValidator()

    birthday = DateField(blank=True, null=True)
    first_name = NameField(verbose_name=_('имя'))
    last_name = NameField(verbose_name=_('фамилия'))
    middle_name = NameField(verbose_name=_('отчетво'))
    usergroup = ManyToManyField(UserGroup, verbose_name=_('группы'))
    username = CodeStrictField(verbose_name=_('логин'), unique=True, validators=[username_validator], error_messages={'unique': _("Такой пользователь уже существует."), }, )

    @property
    def is_admin(self):
        res = False
        for group in self.usergroup.all():
            if group.is_admin:
                res = True
                break
        return res

    @property
    def is_develop(self):
        res = False
        for group in self.usergroup.all():
            if group.is_develop:
                res = True
                break
        return res

    EMAIL_FIELD = 'email'
    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email']

    class Meta:
        verbose_name = _('пользователь')
        verbose_name_plural = _('пользователи')
        abstract = True

    def clean(self):
        super().clean()
        self.email = self.__class__.objects.normalize_email(self.email)

    @property
    def get_full_name(self):
        return f'{self.last_name.strip() if self.last_name else self.username} ' \
               f'{self.first_name.strip() if self.first_name else ""} ' \
               f'{self.middle_name.strip() if self.middle_name else ""}'

    @classmethod
    def full_name(cls, first_name, middle_name, last_name, birthday=None, translit=False):
        if first_name is None and middle_name is None and last_name is None:
            raise NonamedUserException('first_name is None and middle_name is None and last_name is None')

        res = f'{last_name.strip()}' \
              f'{first_name.strip() if first_name else ""} ' \
              f'{middle_name.strip() if middle_name else ""} ' \
              f'{birthday if birthday else ""}'
        if translit is True:
            return AuditModel.translit(res.strip())
        return res.strip()

    @classmethod
    def short_name(cls, first_name, middle_name, last_name, translit=False):
        if first_name is None and middle_name is None and last_name is None:
            raise Exception('first_name is None and middle_name is None and last_name is None')

        from isc_common.models.audit import AuditModel
        fn = f'{first_name.strip()[0:1].upper() if first_name else ""}'
        if fn != '':
            fn = ' ' + fn + '.'

        mn = f'{middle_name.strip()[0:1].upper() if middle_name else ""}'
        if mn != '':
            mn = ' ' + mn + '.'

        if last_name is not None:
            res = f'{last_name.strip()} {fn}{mn}'
        else:
            res = f'{fn}{mn}'
        if translit is True:
            return AuditModel.translit(res.strip())
        return res.strip()

    @property
    def get_short_name(self):
        fn = f'{self.first_name.strip()[0:1].upper() if self.first_name else ""}'
        if fn != '':
            fn = ' ' + fn + '.'

        mn = f'{self.middle_name.strip()[0:1].upper() if self.middle_name else ""}'
        if mn != '':
            mn = ' ' + mn + '.'

        res = f'{self.last_name.strip() if self.last_name else self.username}{fn}{mn}'
        if res.strip() == "":
            res = self.username
        return res.strip()

    @property
    def get_short_name1(self):
        res = f'{self.first_name.strip()[0:1].upper() if self.first_name else ""} ' \
              f'{self.middle_name.strip()[0:1].upper() if self.middle_name else ""}'
        if res.strip() == "":
            res = self.username
        return res.strip()

    def user_short_name(self):
        return self.get_short_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Send an email to this user."""
        send_mail(subject, message, from_email, [self.email], **kwargs)



import logging

from django.db.models import URLField

from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class LinksQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Model_linksQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        link = defaults.get('link')

        if link != '' and link is not None:
            link_item = Links.objects.getOptional(link=link)
            if link_item is None:
                try:
                    item = super().get(**kwargs)
                    link_item, _ = Links.objects.update_or_create(id=item.link.id, defaults=dict(link=link))
                except self.model.DoesNotExist:
                    link_item = Links.objects.create(link=link)
            defaults = dict(link=link_item)
            return super().update_or_create(defaults=defaults, **kwargs)
        else:
            return None, None


class LinksManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return LinksQuerySet(self.model, using=self._db)


class Links(AuditModel):
    link = URLField(db_index=True, unique=True)

    objects = LinksManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Якоря'



# Generated by Django 2.2.4 on 2019-08-21 20:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0019_delete_holidays_view'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='shift_day',
            name='props',
        ),
    ]



import logging
import os
from shutil import copyfile

from django.conf import settings
from django.core.files import File
from django.db.models import BinaryField, BigIntegerField, CharField, TextField
from django.forms import model_to_dict

from isc_common import delAttr
from isc_common.common.UploadItemEx import UploadItemEx
from isc_common.fields.files import FileFieldEx
from isc_common.fields.name_field import NameField
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel

logger = logging.getLogger(__name__)


class CryptoQuerySet(CommonManagetWithLookUpFieldsQuerySet):

    def delete(self):
        for item in self:
            Crypto_file.remove_file(item=item)
        return super().delete()


class CryptoManager(CommonManagetWithLookUpFieldsManager):
    def createFromRequest(self, request, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        for key in data:
            if key.find('__') != -1:
                delAttr(_data, key)
        delAttr(_data, 'form')
        if data.get('id') or not data.get('real_name'):
            delAttr(_data, 'id')
            res = super().filter(id=data.get('id')).update(**_data)
            res = model_to_dict(res[0])
            delAttr(res, 'attfile')
            delAttr(res, 'form')
            data.update(res)
        return data

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        delAttr(data, 'form')
        super().filter(id=request.get_id()).update(**data)
        return data

    def get_queryset(self):
        return CryptoQuerySet(self.model, using=self._db)


class Crypto_file(AuditModel):
    format = NameField(verbose_name='Формат файла')
    mime_type = NameField(verbose_name='MIME тип файла файла')
    size = BigIntegerField(verbose_name='Размер фала', default=0)
    real_name = TextField(verbose_name='Первоначальное имя файла', db_index=True, unique=True)
    key = BinaryField(max_length=200, null=True, blank=True)
    attfile = FileFieldEx(verbose_name='Файл', max_length=255)
    file_store = CharField(verbose_name='Каталог хранения файла', max_length=255, null=True, blank=True)

    object = CryptoManager()

    @property
    def olnly_real_file_name(self):
        ls = self.real_name.split('\\')
        return ls[len(ls) - 1]

    @classmethod
    def remove_file(cls, item):
        old_file_store = item.file_store
        file_path = item.attfile.name

        if old_file_store:
            file_path = file_path.replace(old_file_store, settings.FILES_STORE)

        if isinstance(settings.REPLACE_FILE_PATH, dict):
            for key, value in settings.REPLACE_FILE_PATH.items():
                file_path = file_path.replace(key, value)

        if file_path:
            if os.altsep:
                file_path = file_path.replace(os.altsep, os.sep)

            if os.path.exists(file_path):
                res = os.remove(file_path)
                logger.debug(f'Removed file: {file_path}')
            else:
                logger.warning(f'Removed file: {file_path} not finded.')

    @classmethod
    def copy_file(cls, item):
        if os.path.exists(item.real_file_name):
            copyfile(src=item.real_file_name, dst=item.full_path)
        else:
            copyfile(src=item.tmp_file_name, dst=item.full_path)
        with open(item.full_path, 'rb') as src:
            fileObj = File(src)
            res, created = cls.objects.update_or_create(
                id=item.id,
                defaults=dict(
                    attfile=fileObj,
                    file_store=item.get_path(fileObj.name),
                    format=item.file_format,
                    mime_type=item.file_mime_type,
                    size=item.file_size,
                    real_name=item.real_file_name,
                )
            )

        if os.path.exists(item.tmp_file_name):
            os.remove(item.tmp_file_name)

        return res, created

    @classmethod
    def get_action(cls, res, item):
        action = None

        if res.size is None or res.size == 0 or not os.path.exists(str(res.attfile)):
            action = 'create'
        elif res.size != item.file_size:
            action = 'update'
        return action

    @classmethod
    def create_update(cls, **kwargs):
        item = UploadItemEx(**kwargs)

        try:
            if item.id is not None:
                res = cls.objects.get(id=item.id)
                action = cls.get_action(res=res, item=item)
            else:
                if item.real_file_name is None:
                    raise Exception('Must be real_file_name or id')
                else:
                    res = cls.objects.get(real_name=item.real_file_name)

                action = cls.get_action(res=res, item=item)

            if action is not None:
                if action == 'update':
                    cls.remove_file(item=res)

                return cls.copy_file(item=item)
            else:
                return None, None

        except cls.DoesNotExist:
            return cls.copy_file(item=item)

    def __str__(self):
        return f"{self.real_name}"

    class Meta:
        abstract = True



# Generated by Django 3.1.7 on 2021-03-15 11:54

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0085_user_e_mails_user_phones'),
    ]

    operations = [
        migrations.AddField(
            model_name='user_e_mails',
            name='props',
            field=bitfield.models.BitField((('main', 'Главная'),), db_index=True, default=0),
        ),
        migrations.AddField(
            model_name='user_phones',
            name='props',
            field=bitfield.models.BitField((('main', 'Главная'),), db_index=True, default=0),
        ),
    ]



import logging

from bitfield import BitField
from crypto.models.crypto_file import Crypto_file
from django.db.models import DateTimeField

logger = logging.getLogger(__name__)


class Files_1c(Crypto_file):
    file_modification_time = DateTimeField(verbose_name='Дата время поcледнего модификации документа')
    props = BitField(flags=(('imp', 'Импорт'), ('exp', 'Экспорт')), db_index=True)

    def __str__(self):
        return f"id: {self.id}, real_name: {self.real_name}"

    class Meta:
        verbose_name = 'Файлы 1С'



# Generated by Django 2.2.4 on 2019-08-25 10:15

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0057_auto_20190825_1014'),
    ]

    operations = [
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='saturday',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='saturday_rel', to='clndr.Shift_day'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-16 04:50

from django.db import migrations
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0008_auto_20190610_1556'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='files_1c',
            managers=[
                ('object', django.db.models.manager.Manager()),
            ],
        ),
    ]



import os
from typing import Text
from uuid import UUID


def getOrElse(default, value):
    if value:
        return value
    else:
        return default


black = "black"
blue = "blue"
closed = "closed"
create = "create"
deleted = "deleted"
developers = "developers"
doing = "doing"
execution = "execution"
formirovanie = "formirovanie"
gray = "#808080"
green = "green"
handmade = "handmade"
in_production = "in_production"
launched = "launched"
light_green = "#99CC00"
made = "made"
new = "new"
new_man = "new_man"
not_relevant = "not_relevant"
orange = "orange"
otkryt = "otkryt"
otmenen = "otmenen"
red = "red"
notDefined = "Не определен"
restarted = "restarted"
route_made = "route_made"
route_made_error = "route_made_error"
sht = "шт"
started = "started"
started_another = "started_another"
stoped = "stoped"
transferred = "transferred"
unknown = "unknown"
undefined = "undefined"
update = "update"
value_odd = "value_odd"
zakryt = "zakryt"

name_closed = "Закрыт"
name_doing = "Выполнен"
name_formirovanie = "Формирование"
name_handmade = "Системный (ручное формирование)"
name_new = "Новый"
name_new_h = "Новый (р)"
name_new_s = "Новый (с)"
name_restarted = "Запущен (повторно)"
name_started = "Запущен"
name_started_another = "Запущен (раннее)"
name_stoped = "Остановлен"
name_transferred = "Назначенный"
name_unknown = "Неопределеный"


def blinkString(text, blink=True, color="black", bold=False) -> Text:
    if blink:
        res = f'<div class="blink"><font color="{color}"</font>{text}</div>'
    else:
        res = f'<div><font color="{color}"</font>{text}</div>'

    if bold == True:
        return f'<b>{res}</b>'
    else:
        return res


def blinkString1(text, blink=True, color="black", bold=True) -> Text:
    if blink:
        res = f'<blink class="blink"><font color="{color}"</font>{text}</blink>'
    else:
        res = f'<blink><font color="{color}"</font>{text}</blink>'

    if bold == True:
        return f'<b>{res}</b>'
    else:
        return res


def to_H(text, level=4):
    return f'<h{level}>{text}</h{level}>'


def uuid5():
    return UUID(bytes=os.urandom(16), version=4)


def uuid4():
    return str(UUID(bytes=os.urandom(16), version=4)).upper().replace('-', '_')



# Generated by Django 2.2.3 on 2019-07-02 12:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('refs', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='type_param',
            name='exponent',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='type_param',
            name='length',
            field=models.PositiveIntegerField(blank=True, default=255, null=True),
        ),
        migrations.AlterField(
            model_name='type_param',
            name='precision',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]



import logging

from django.db.models import Model

from isc_common.fields.code_field import CodeField
from isc_common.fields.name_field import NameField

logger = logging.getLogger(__name__)


class Pg_class(Model):
    relname = CodeField(primary_key=True, null=False)
    relkind = NameField()

    def __str__(self):
        return f"{self.relname}"

    class Meta:
        verbose_name = 'pg_class'
        managed = False;
        db_table = 'pg_class'



import logging

from isc_common.auth.models.user import User, UserManager

logger = logging.getLogger(__name__)


class UserView(User):
    objects = UserManager()

    class Meta:
        verbose_name = 'Пользователь'
        managed = False
        db_table = 'isc_common_user_view'



import logging
import random

from django.db.models import PositiveIntegerField
from django.forms import model_to_dict

from clndr.models.shift_day import Shift_day
from isc_common import setAttr, delAttr
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyCascade
from isc_common.fields.time_field import TimeField
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet

logger = logging.getLogger(__name__)


class Shift_day_relaxQuerySet(BaseRefQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Shift_day_relaxManager(BaseRefManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'begintime': record.begintime,
            'endtime': record.endtime,
            'shiftday_id': record.shiftday.id,
            'color': record.color,
            'relaxorder': record.relaxorder,
        }
        return res

    def get_queryset(self):
        return Shift_day_relaxQuerySet(self.model, using=self._db)

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        setAttr(data, 'id', random.randint(1, 100000000))
        return data

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        return request.get_data()

    # def deleteFromRequest(self, request):
    #     request = DSRequest(request=request)
    #     return 1


class Shift_day_relax(BaseRef):
    color = CodeField()
    relaxorder = PositiveIntegerField()
    shiftday = ForeignKeyCascade(Shift_day)

    begintime = TimeField(verbose_name='Начало периода', db_index=True)
    endtime = TimeField(verbose_name='Конец периода', db_index=True)

    def copy(self):
        data = model_to_dict(Shift_day_relax.objects.get(id=self.id))
        delAttr(data, 'id')
        return Shift_day_relax.objects.create(**data)

    @property
    def period_html(self):
        return f'<p><u>{self.begintime} - {self.endtime}</u></p>'

    @property
    def html(self):
        descr = '' if self.description is None else f'({self.description})'
        return f'<div><font color="{self.color}"><p><b>{self.name} {descr}</b></p>{self.period_html}</font></div>'

    objects = Shift_day_relaxManager()

    def __str__(self):
        return f"ID={self.id}, " \
            f"code={self.code}, " \
            f"name={self.name}, " \
            f"description={self.description}, " \
            f"color={self.color}, " \
            f"shiftday=[{self.shiftday}]"

    class Meta:
        verbose_name = 'Запланированные перекуры и отдыхи'
        ordering = ['relaxorder', 'shiftday']
        unique_together = (('relaxorder', 'shiftday'),)



from django.urls import path

from isc_common.views import text_informations

urlpatterns = [

    path('Text_informations/Fetch/', text_informations.Text_informations_Fetch),
    path('Text_informations/Add', text_informations.Text_informations_Add),
    path('Text_informations/Update', text_informations.Text_informations_Update),
    path('Text_informations/Remove', text_informations.Text_informations_Remove),
    path('Text_informations/Lookup/', text_informations.Text_informations_Lookup),
    path('Text_informations/Info/', text_informations.Text_informations_Info),
    path('Text_informations/Copy', text_informations.Text_informations_Copy),

]



# Generated by Django 3.1.7 on 2021-03-15 09:22

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0082_auto_20210312_0927'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='users_images',
            name='club',
        ),
    ]



def get_NoneStr(str):
    if str is None:
        return ''
    else:
        return str


def get_NullStr(str):
    if str is None:
        return ''
    elif str == 'null':
        return ''
    else:
        return str

def StrToBytes(mystring):
    if isinstance(mystring, str):
        return mystring.encode('utf-8')
    else:
        raise Exception(f'{mystring} must be str')


def BytesToStr(mybytes):
    if isinstance(mybytes, bytes):
        return mybytes.decode('utf-8')
    else:
        raise Exception(f'{mybytes} must be bytes')



import logging
from tempfile import TemporaryFile

from django.core.files import File
from django.db import transaction

from crypto.models.upload_file import DSResponse_UploadFile
from isc_common.auth.models.user import User
from isc_common.common import uuid4
from isc_common.common.UploadItem import UploadItem
from isc_common.common.functions import Common
from isc_common.dropzone import Dz
from isc_common.http.RPCResponse import RPCResponseConstant

logger = logging.getLogger(__name__)


class DSResponse__Messages_UploadFile1(DSResponse_UploadFile):
    user = None

    def __init__(self, request):

        self.dz_dictionary = Dz(request.POST)

        self.host = self.dz_dictionary.host
        self.port = self.dz_dictionary.port
        self.channel = self.dz_dictionary.ws_channel

        self.file = request.FILES.get('file')
        self.user = User.objects.get(id=self.dz_dictionary.user_id)

        setattr(self.dz_dictionary, 'real_file_name', self.file.name)
        setattr(self.dz_dictionary, 'stored_file_name', self.dz_dictionary.dzuuid)
        setattr(self.dz_dictionary, 'file_size', self.dz_dictionary.dztotalfilesize)
        setattr(self.dz_dictionary, 'file_mime_type', self.file.content_type)
        setattr(self.dz_dictionary, 'guid', uuid4())

        self.handle_uploaded_file()

    @property
    def first_chunk(self):
        res = int(self.dz_dictionary.dzchunkindex) == 0
        return res

    @property
    def last_chunk(self):
        res = int(self.dz_dictionary.dztotalchunkcount) == int(self.dz_dictionary.dzchunkindex) + 1
        return res

    def handle_uploaded_file(self):
        try:

            item = UploadItem(dictionary=self.dz_dictionary.__dict__)

            def load_str(pers):
                return f'Загружено: {self.file.name}: {pers} %'

            with TemporaryFile() as src:
                src.seek(int(self.dz_dictionary.dzchunkbyteoffset))
                src.write(self.file.read())

                pers = round((int(self.dz_dictionary.dzchunkindex) * 100) / int(self.dz_dictionary.dztotalchunkcount), 2)
                if self.last_chunk:
                    pers = 100

                logger.debug(f'{load_str(pers)}, шаг : {int(self.dz_dictionary.dzchunkindex) + 1} из {self.dz_dictionary.dztotalchunkcount}')

                if self.last_chunk:
                    logger.debug(f'Загружен файл: {item.real_file_name}')

                    logger.debug(f'Запись временного файла.')
                    with open(item.full_path, 'w+b') as destination:
                        src.seek(0)
                        destination.write(src.read())
                    logger.debug(f'Запись временного файла выполнена.')

                    logger.debug(f'Запись: {item.full_path}.')
                    with open(item.full_path, 'rb') as src:
                        fileObj = File(src)

                        with transaction.atomic():
                            from tracker.models.messages_files import Messages_files
                            messages_file = Messages_files.objects.create(
                                attfile=fileObj,
                                file_store=self.get_path(fileObj.name),
                                format=item.file_format,
                                mime_type=item.file_mime_type,
                                real_name=item.real_file_name,
                                size=item.file_size,
                            )

                            url = f'logic/Messages_files/Download/{messages_file.id}/?host={self.host}&port={self.port}&ws_channel=common_{self.user.username}'
                            message = f'<p/><div>Выложен файл: <a href="{url}" target="hiddenframe"><strong>{item.real_file_name}</strong></a>   {Common.get_size_file_str(item.file_size)}</div><p/>'

                            face = "times new roman,times,serif"
                            size = 4

                            message = f'<font face ="{face}" size="{size}">{message}</font>'

                            logger.debug(f'Запись: {item.real_file_name} ({fileObj.name}) завершена.')

                            self.remove(item.full_path)
                            logger.debug(f'Удаление: {item.full_path} завершено.')

                    self.response = dict(response=dict(data=dict(message=message, id=messages_file.id)), status=RPCResponseConstant.statusSuccess)

        except Exception as e:
            raise e



import logging

from django.db import transaction

from history.models.visitor import Visitor
from isc_common.http.DSRequest import DSRequest
from isc_common.http.RPCResponse import RPCResponseConstant

logger = logging.getLogger(__name__)


class LogoutRequest(DSRequest):
    def __init__(self, request):
        DSRequest.__init__(self, request)
        data = self.get_data()

        with transaction.atomic():
            Visitor.objects.using('history').select_for_update()
            Visitor.objects.using('history').filter(username=data.get('login')).delete()

        self.response = dict(status=RPCResponseConstant.statusSuccess)



class Dz:
    dzchunkbyteoffset = None
    dzchunkindex = None
    dztotalchunkcount = None
    dztotalfilesize = None
    dzuuid = None

    def __init__(self, POST):
        for k, v in POST.dict().items():
            try:
                setattr(self, k, int(v))
            except:
                setattr(self, k, v)



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from twits.models.chat_user_user_theme import Chat_user_user_theme, Chat_user_user_themeManager


@JsonResponseWithException()
def Chat_user_user_theme_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Chat_user_user_theme.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Chat_user_user_themeManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Chat_user_user_theme_Add(request):
    return JsonResponse(DSResponseAdd(data=Chat_user_user_theme.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Chat_user_user_theme_Update(request):
    return JsonResponse(DSResponseUpdate(data=Chat_user_user_theme.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Chat_user_user_theme_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Chat_user_user_theme.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Chat_user_user_theme_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Chat_user_user_theme.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Chat_user_user_theme_Info(request):
    return JsonResponse(DSResponse(request=request, data=Chat_user_user_theme.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Chat_user_user_theme_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Chat_user_user_theme.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.1.7 on 2021-03-12 09:27

import django.db.models.deletion
from django.conf import settings
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0081_auto_20210310_0452'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='auditmodelex',
            name='updator',
        ),
        migrations.AddField(
            model_name='auditmodelex',
            name='editor',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='AuditModelEx_editor', to=settings.AUTH_USER_MODEL),
        ),
    ]



import os
import uuid
from pathlib import Path

import magic
from isc_common.common.UploadItem import UploadItem

class UploadItemEx(UploadItem):

    def get_path(self, name):
        return os.path.dirname(os.path.abspath(name)).replace(os.sep, os.altsep) if os.altsep else os.path.dirname(os.path.abspath(name))

    def __init__(self, logger=None, **kwargs):
        super().__init__(logger=logger, dictionary=kwargs)

        if self.real_file_name is not None and self.file_mime_type is None:
            mime = magic.Magic(mime=True)
            if os.path.exists(self.real_file_name):
                self.file_mime_type = mime.from_file(self.real_file_name)
                stat = Path(self.real_file_name).stat()
            else:
                self.file_mime_type = mime.from_file(self.tmp_file_name)
                stat = Path(self.tmp_file_name).stat()


        if self.file_size is None:
            self.file_size = stat.st_size

        if self.stored_file_name is None:
            self.stored_file_name = str(uuid.uuid4()).upper()



import logging

from django.db.models import BigIntegerField
from isc_common.fields.code_field import CodeStrictField
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class Edited_byQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Edited_byManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Edited_byQuerySet(self.model, using=self._db)


class Edited_by(AuditModel):
    table_id = BigIntegerField()
    table_name = CodeStrictField()
    objects = Edited_byManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Таблица сделанных изменений'



# Generated by Django 3.1.6 on 2021-03-01 11:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0073_auto_20210301_1040'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='real_name',
            field=models.TextField(blank=True, db_index=True, null=True, verbose_name='Первоначальное имя файла'),
        ),
    ]



from django.urls import path

import twits
from twits.views.chats import Chats_Fetch, Chats_Add, Chats_Update, Chats_Remove, Chats_Lookup, Chats_Info, Chats_Copy

urlpatterns = [

    path('Chats/Fetch/', Chats_Fetch),
    path('Chats/Add', Chats_Add),
    path('Chats/Update', Chats_Update),
    path('Chats/Remove', Chats_Remove),
    path('Chats/Lookup/', Chats_Lookup),
    path('Chats/Info/', Chats_Info),
    path('Chats/Copy', Chats_Copy),

]



# Generated by Django 3.1.3 on 2020-11-20 13:16

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0055_auto_20200713_0552'),
    ]

    operations = [
        migrations.CreateModel(
            name='Last_used_processes',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('key', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('process', models.BigIntegerField(db_index=True)),
                ('user', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Сохраненные параметры',
                'unique_together': {('user', 'key', 'process')},
            },
        ),
    ]



import logging
import sys
from typing import Text

from django.conf import settings
from django.db import connection

from isc_common.common import uuid4
from sheduler import make_shedurer_tasks

logger = logging.getLogger(__name__)


def exists_base_object(table_name, schem_aname='public'):
    sql_text = '''SELECT n.nspname, c.relname
                    FROM pg_catalog.pg_class c
                             JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
                    WHERE c.relname = %s
                      AND n.nspname = %s'''

    with connection.cursor() as cursor:
        cursor.execute(sql_text, [table_name, schem_aname])
        rows = cursor.fetchone()
        if rows is None:
            return False
        else:
            return len(rows) > 0


def create_tmp_mat_view(sql_str, params=[], indexes=[], mat_view_name=None, check_exists=False) -> Text:
    if check_exists == True and check_exists is not None and exists_base_object(mat_view_name):
        return mat_view_name

    key = f'create_tmp_mat_view_{mat_view_name}'
    if mat_view_name is not None:
        settings.LOCKS.acquire(key)
    else:
        mat_view_name = f'tmp_{uuid4()}'

    if isinstance(indexes, list):
        indexes = [f'''CREATE INDEX {mat_view_name}_{index}_idx ON {mat_view_name} USING btree ("{index}")''' for index in indexes]
        suffix = ';'.join(indexes)
    sql_txt = f'''CREATE MATERIALIZED VIEW {mat_view_name} AS {sql_str} WITH DATA; {suffix}'''
    try:
        with connection.cursor() as cursor:
            logger.debug(f'Creating: {mat_view_name}')
            cursor.execute(sql_txt, params)
            logger.debug(f'Created: {mat_view_name}')
        if mat_view_name is not None:
            settings.LOCKS.release(key)
        return mat_view_name
    except Exception as ex:
        exc_info = sys.exc_info()
        logger.error(msg=ex, exc_info=exc_info)
        if mat_view_name is not None:
            settings.LOCKS.release(key)
        return None


def create_view(sql_str, params=[], view_name=None) -> None:
    if view_name is None:
        raise Exception(f'View must have name')

    if exists_base_object(view_name):
        return view_name

    key = f'createt_view_{view_name}'
    settings.LOCKS.acquire(key)

    sql_txt = f'''DROP VIEW IF EXISTS {view_name} CASCADE;'''
    sql_txt += f'''CREATE VIEW {view_name} AS {sql_str};'''
    try:
        with connection.cursor() as cursor:
            logger.debug(f'Creating: {view_name}')
            cursor.execute(sql_txt, params)
            logger.debug(f'Created: {view_name}')
        settings.LOCKS.release(key)
    except Exception as ex:
        exc_info = sys.exc_info()
        logger.error(msg=ex, exc_info=exc_info)
        settings.LOCKS.release(key)


def create_insert_update_delete_function_of_table(
        table_name,
        table_schema='public',
        exclude_fields=[],
        func_params=[('id', 'bigint')],
        pk_names=['id']
):
    settings.LOCKS.acquire(settings.GRID_CONSTANTS.lock_insert_update_delete_function_of_table)
    try:
        columns_sql_str = f'''select column_name, data_type, ordinal_position
                                    from information_schema.columns
                                        where table_schema = %s
                                    and table_name = %s
                                    order by ordinal_position'''

        def insert_proc_body_str(fields, where_clause):
            return f'''insert into {table_name}_tbl ({fields}) select {fields} from {table_name}_view where {where_clause}'''

        with connection.cursor() as cursor:
            cursor.execute(columns_sql_str, [table_schema, f'{table_name}_view'])
            all_rows = cursor.fetchall()

            pk_names.extend(exclude_fields)
            rows_without_pk_field = [row for row in all_rows if row[0] not in pk_names]

            def func_param_str(param_item):
                return f'{param_item[0]} {param_item[1]}'

            def func_where_clause_str(param_item, prefix=None):
                return f'''{f'{prefix}.' if prefix else ''}"{param_item[0]}"=f{param_item[0]}'''

            def func_where_clause_str1(param_item, prefix1=None, prefix2=None):
                prefix2 = '' if prefix2 is None else f'{prefix2}.'
                return f'''{f'{prefix1}.' if prefix1 else ''}"{param_item[0]}"={prefix2}"{param_item[0]}"'''

            func_params1 = ','.join(f'f{func_param_str(a)}' for a in func_params)
            func_params2 = ','.join(f'f{a[0]}' for a in func_params)
            func_params_types = ','.join(f'{a[1]}' for a in func_params)

            where_clause = ' and '.join(f'{func_where_clause_str(a)}' for a in func_params)
            where_clause_prefix_s = ' and '.join(f'''{func_where_clause_str(param_item=a, prefix='s')}''' for a in func_params)
            where_clause_prefix_tbl = ' and '.join(f'''{func_where_clause_str1(param_item=a, prefix1=f'{table_name}_tbl', prefix2='s')}''' for a in func_params)

            fields = ','.join([f'"{row[0]}"' for row in all_rows])
            fields_without_pk_field = ','.join([f'"{row[0]}"' for row in rows_without_pk_field])

            fields_update = ','.join([f'"{row[0]}"=s."{row[0]}"' for row in rows_without_pk_field])

            insert_func_name = f'insert_into_{table_name}'
            insert_proc_str = f'''drop function if exists {insert_func_name}({func_params_types}) cascade;
                                      create function {insert_func_name}({func_params1}) returns bigint
                                            language plpgsql
                                        as
                                        $$
                                        BEGIN
                                            {insert_proc_body_str(fields=fields, where_clause=where_clause)};
                                            return fid;
                                        END;
                                        $$;'''

            cursor.execute(insert_proc_str)

            insert_proc_str = f'''drop function if exists {insert_func_name}_s(bigint[]) cascade;
                                                  create function {insert_func_name}_s(ids bigint[]) returns bigint
                                                        language plpgsql
                                                    as
                                                    $$
                                                    BEGIN
                                                        {insert_proc_body_str(fields=fields, where_clause='id=any(ids)')};
                                                        return 0;
                                                    END;
                                                    $$;'''

            cursor.execute(insert_proc_str)

            delete_proc_str = f'''drop function if exists delete_{table_name}_s(bigint[]) cascade;
                                                            create function delete_{table_name}_s(ids bigint[]) returns bigint
                                                                language plpgsql
                                                            as
                                                            $$
                                                            BEGIN
                                                                delete from {table_name}_tbl where id=any(ids);
                                                                return 0;
                                                            END;
                                                            $$;
                                                            '''

            cursor.execute(delete_proc_str)

            update_proc_str = f'''drop function if exists update_{table_name}_s(bigint[]) cascade;
                                            create function update_{table_name}_s(ids bigint[])  RETURNS bigint
                                                language plpgsql
                                            as
                                            $$
                                            BEGIN
                                                update {table_name}_tbl
                                                set 
                                                   {fields_update}
                                                from (select {fields}
                                                      from {table_name}_view) as s
                                                        where  {where_clause_prefix_tbl}
                                                        and s.id = any(ids);
                                                        return 0;                                        

                                            END;
                                            $$;'''

            cursor.execute(update_proc_str)

            update_proc_str = f'''drop function if exists update_{table_name}({func_params_types}) cascade;
                                create function update_{table_name}({func_params1})  RETURNS bigint
                                    language plpgsql
                                as
                                $$
                                declare
                                    cnt numeric;
                                BEGIN
                                
                                    select count(*)
                                    into cnt
                                    from {table_name}_tbl as s 
                                    where {where_clause_prefix_s};
                                
                                    if cnt = 0 then
                                        return {insert_func_name}({func_params2});
                                    else
                                        update {table_name}_tbl
                                        set 
                                           {fields_update}
                                        from (select {fields}
                                              from {table_name}_view) as s
                                                where  {where_clause_prefix_tbl}
                                                and {where_clause_prefix_s};
                                                return fid;                                        
                                    end if;
                                   
                                END;
                                $$;'''

            cursor.execute(update_proc_str)

            delete_proc_str = f'''drop function if exists delete_{table_name}({func_params_types}) cascade;
                                    create function delete_{table_name}({func_params1}) returns void
                                        language plpgsql
                                    as
                                    $$
                                    BEGIN
                                        delete from {table_name}_tbl where {where_clause};
                                    END;
                                    $$;
                                    '''

            cursor.execute(delete_proc_str)

            settings.LOCKS.release(settings.GRID_CONSTANTS.lock_insert_update_delete_function_of_table)
    except Exception as ex:
        settings.LOCKS.release(settings.GRID_CONSTANTS.lock_insert_update_delete_function_of_table)
        logger.error(ex)


def create_table(
        fields_str=None,
        sql_str=None,
        temporary='',
        on_commit=None,
        params=[],
        indexes=[],
        unique_indexes=[],
        primary_key=None,
        table_name=None,
        any_constraints=[],
        drop=True) -> None:
    if table_name is None:
        raise Exception(f'Table must have name')

    key = f'create_tmp_table{table_name}'
    settings.LOCKS.acquire(key)

    pk_str = ''
    if primary_key is not None:
        if not isinstance(primary_key, list):
            primary_key = [primary_key]

        # pk_str = f'''ALTER TABLE {table_name}
        #             ADD CONSTRAINT {table_name}_pk
        #                 PRIMARY KEY ({','.join(primary_key)});'''

        pk_str = f'''ALTER TABLE {table_name}
                            ADD CONSTRAINT {table_name}_pk
                                UNIQUE ({','.join(primary_key)});'''

    indexes_str = ''
    if isinstance(indexes, list):
        indexes = [f'''CREATE INDEX {table_name}_{index}_idx ON {table_name} USING btree ("{index}")''' for index in indexes]
        indexes_str = ';'.join(indexes)

    unique_indexes_str = ''
    if isinstance(unique_indexes, list):
        if len(unique_indexes):
            qt_fld = list(map(lambda x: f'"{x}"', unique_indexes[0:len(unique_indexes) - 1]))
            unique_indexes_str = ';\n' + f'''CREATE UNIQUE INDEX {table_name}_{'_'.join(unique_indexes)}_idx ON {table_name} USING btree ({','.join(qt_fld)}) WHERE ("{unique_indexes[len(unique_indexes) - 1]}" is null);'''

    sql_txt = ''
    if drop:
        sql_txt = f'''DROP TABLE IF EXISTS {table_name} CASCADE;'''

    on_commit_str = f'ON COMMIT {on_commit}'
    if on_commit is None:
        on_commit_str = ''

    if sql_str is None:
        sql_txt += f'''CREATE {temporary} TABLE {table_name} ({fields_str}) {on_commit_str}'''
    else:
        sql_txt += f'''CREATE {temporary} TABLE {table_name} as ({sql_str}); {on_commit_str}'''

    sql_txt += pk_str
    sql_txt += ';\n'.join(any_constraints)
    sql_txt += indexes_str
    sql_txt += unique_indexes_str

    try:
        with connection.cursor() as cursor:
            logger.debug(f'Creating: {table_name}')
            cursor.execute(sql_txt, params)
            logger.debug(f'Created: {table_name}')
        settings.LOCKS.release(key)
    except Exception as ex:
        exc_info = sys.exc_info()
        logger.error(msg=ex, exc_info=exc_info)
        settings.LOCKS.release(key)
        raise ex


def create_table1(
        fields_str=None,
        sql_str=None,
        temporary='',
        on_commit=None,
        params=[],
        indexes=[],
        unique_indexes=[],
        primary_key=None,
        table_name=None,
        any_constraints=[],
        drop=True) -> None:
    if table_name is None:
        raise Exception(f'Table must have name')

    key = f'create_tmp_table{table_name}'
    settings.LOCKS.acquire(key)

    pk_str = ''
    if primary_key is not None:
        if not isinstance(primary_key, list):
            primary_key = [primary_key]

        # pk_str = f'''ALTER TABLE {table_name}
        #             ADD CONSTRAINT {table_name}_pk
        #                 PRIMARY KEY ({','.join(primary_key)});'''

        pk_str = f'''ALTER TABLE {table_name}
                            ADD CONSTRAINT {table_name}_pk
                                UNIQUE ({','.join(primary_key)});'''

    indexes_str = ''
    if isinstance(indexes, list):
        indexes = [f'''CREATE INDEX {table_name}_{index}_idx ON {table_name} USING btree ("{index}")''' for index in indexes]
        indexes_str = ';'.join(indexes)

    unique_indexes_str = ''
    if isinstance(unique_indexes, list):
        if len(unique_indexes):
            qt_fld = list(map(lambda x: f'"{x}"', unique_indexes[0:len(unique_indexes) - 1]))
            unique_indexes_str = ';\n' + f'''CREATE UNIQUE INDEX {table_name}_{'_'.join(unique_indexes)}_idx ON {table_name} USING btree ({','.join(qt_fld)}) WHERE ("{unique_indexes[len(unique_indexes) - 1]}" is null);'''

    sql_txt = ''
    if drop:
        sql_txt = f'''DROP TABLE IF EXISTS {table_name} CASCADE;'''

    on_commit_str = f'ON COMMIT {on_commit}'
    if on_commit is None:
        on_commit_str = ''

    if sql_str is None:
        sql_txt += f'''CREATE {temporary} TABLE {table_name} ({fields_str}) {on_commit_str};'''
    else:
        sql_txt += f'''CREATE {temporary} TABLE {table_name} as ({sql_str}); {on_commit_str};'''

    sql_txt += pk_str
    sql_txt += ';\n'.join(any_constraints)
    sql_txt += indexes_str
    sql_txt += unique_indexes_str

    try:

        with connection.cursor() as cursor:
            logger.debug(f'Creating: {table_name}')
            cursor.execute(sql_txt, params)
            logger.debug(f'Created: {table_name}')
            view_name = table_name.replace("tbl", "view")

            sql_txt = f'insert into {table_name} select * from {view_name} where id=%s'

            sql_txt1 = f'select count(*) from planing_production_order_lt_view'
            cursor.execute(sql_txt1, params)
            cnt, = cursor.fetchone()
            i = 0

            sql_txt1 = f'select id from planing_production_order_lt_view'

            cursor.execute(sql_txt1, params)
            rows = cursor.fetchall()
            for row in rows:
                logger.debug(f'Inserting ({i} form {cnt}): Where id={row[0]}')
                cursor.execute(sql_txt, row)
                i += 1

        settings.LOCKS.release(key)
    except Exception as ex:
        exc_info = sys.exc_info()
        logger.error(msg=ex, exc_info=exc_info)
        settings.LOCKS.release(key)
        raise ex


def create_tmp_table(fields_str=None, sql_str=None, on_commit='delete rows', params=[], indexes=[], unique_indexes=[], table_name=None, drop=True) -> None:
    create_table(
        drop=drop,
        fields_str=fields_str,
        indexes=indexes,
        on_commit=on_commit,
        params=params,
        sql_str=sql_str,
        table_name=table_name,
        temporary='TEMPORARY',
        unique_indexes=unique_indexes,
    )


def drop_mat_view(mat_view_name) -> bool:
    if not exists_base_object(mat_view_name):
        return False

    sql_txt = f'DROP MATERIALIZED VIEW IF EXISTS {mat_view_name} CASCADE;'
    try:
        if exists_base_object(mat_view_name) == False:
            return False

        key = f'drop_mat_view_{mat_view_name}'
        settings.LOCKS.acquire(key)
        with connection.cursor() as cursor:
            logger.debug(f'Droping: {mat_view_name}')
            cursor.execute(sql_txt)
        settings.LOCKS.release(key)
        logger.debug(f'Droped: {mat_view_name}')
        return True
    except Exception as ex:
        exc_info = sys.exc_info()
        logger.error(msg=ex, exc_info=exc_info)
        settings.LOCKS.release(key)
        return False


def refresh_mat_view(mat_view_name, username=None):
    from isc_common.ws.webSocket import WebSocket

    if not exists_base_object(mat_view_name):
        return

    def _refresh():
        sql_txt = f'REFRESH MATERIALIZED VIEW {mat_view_name};'
        key = f'refresh_mat_view_{mat_view_name}'
        settings.LOCKS.acquire(key)
        try:
            with connection.cursor() as cursor:
                cursor.execute(sql_txt)
            settings.LOCKS.release(key)

            if username is not None:
                WebSocket.send_info_message(
                    host=settings.WS_HOST,
                    port=settings.WS_PORT,
                    channel=f'common_{username}',
                    message=f'Обновление {mat_view_name} выполнено.',
                    logger=logger
                )

        except Exception as ex:
            exc_info = sys.exc_info()
            logger.error(msg=ex, exc_info=exc_info)
            settings.LOCKS.release(key)

    make_shedurer_tasks(_refresh, when='1s')

    return True



# Generated by Django 3.1.7 on 2021-03-15 11:38

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0083_remove_users_images_club'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='email',
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import connection, IntegrityError
from tqdm import tqdm

from isc_common.models.images import Images
from isc_common.models.users_images import Users_images
from lfl_admin.competitions.models.calendar_images import Calendar_images
from lfl_admin.competitions.models.club_logo_history_images import Club_logo_history_images
from lfl_admin.competitions.models.clubs_images import Clubs_images
from lfl_admin.competitions.models.divisions_images import Divisions_images
from lfl_admin.competitions.models.leagues_images import Leagues_images
from lfl_admin.competitions.models.players_images import Players_images
from lfl_admin.competitions.models.referees_images import Referees_images
from lfl_admin.competitions.models.tournaments_images import Tournaments_images
from lfl_admin.constructions.models.fields_images import Fields_images
from lfl_admin.constructions.models.stadiums_images import Stadiums_images
from lfl_admin.decor.models.banners import Banners
from lfl_admin.decor.models.news_images import News_images
from lfl_admin.inventory.models.clothes_images import Clothes_images
from lfl_admin.region.models.city_images import City_images
from lfl_admin.region.models.region_images import Region_images
from lfl_admin.region.models.region_zone_images import Region_zone_images

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    def update_model(self, model, id, idss):
        try:
            res = model.objects.filter(image_id__in=idss).update(image_id=id)
            # print(f'res: {res}')
        except IntegrityError:
            res = model.objects.filter(image_id__in=idss).delete()
            # print(f'res: {res}')

    def handle(self, *args, **options):
        with connection.cursor() as cursor:
            cursor.execute('''select count(*), real_name
                                    from isc_common_images
                                    group by real_name
                                    having count(*) > 1''')
            rows = cursor.fetchall()
            pbar = tqdm(total=len(rows))

            for row in rows:
                _, real_name = row
                ids = list(map(lambda x: x.id, Images.objects.filter(real_name=real_name)))
                id = ids[0]
                idss = ids[1:]

                self.update_model(model=Banners, id=id, idss=idss)
                self.update_model(model=Calendar_images, id=id, idss=idss)
                self.update_model(model=City_images, id=id, idss=idss)
                self.update_model(model=Clothes_images, id=id, idss=idss)
                self.update_model(model=Club_logo_history_images, id=id, idss=idss)
                self.update_model(model=Clubs_images, id=id, idss=idss)
                self.update_model(model=Divisions_images, id=id, idss=idss)
                self.update_model(model=Fields_images, id=id, idss=idss)
                self.update_model(model=Leagues_images, id=id, idss=idss)
                self.update_model(model=News_images, id=id, idss=idss)
                self.update_model(model=Players_images, id=id, idss=idss)
                self.update_model(model=Referees_images, id=id, idss=idss)
                self.update_model(model=Region_images, id=id, idss=idss)
                self.update_model(model=Region_zone_images, id=id, idss=idss)
                self.update_model(model=Stadiums_images, id=id, idss=idss)
                self.update_model(model=Tournaments_images, id=id, idss=idss)
                self.update_model(model=Users_images, id=id, idss=idss)

                res = Images.objects.filter(id__in=idss).delete()
                # print(f'res: {res}')

                pbar.update()

            pbar.close()



import logging

from django.core.management import BaseCommand

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    def handle(self, *args, **options):
        # record = {'id': 135, 'code': '2020 / 07 / 1', 'name': None, 'date': datetime.datetime(2020, 7, 17, 11, 53, 8), 'description': None, 'parent_id': None, 'demand_id': None, 'demand__code': None, 'demand__date': None, 'item_id': None, 'parent_item_id': None, 'item__STMP_1_id': None, 'item__STMP_1__value_str': None, 'item__STMP_2_id': None, 'item__STMP_2__value_str': None, 'status_id': 7, 'status__code': 'in_production', 'status__name': 'Сформированы заказы на производство', 'qty': '', 'qty_made': 0, 'value_made': 100, 'value_sum': 100, 'isFolder': True, 'priority': '', 'editing': True, 'deliting': True}
        # WebSocket.row_refresh_grid(settings.GRID_CONSTANTS.refresh_production_launch_grid_row, record)

        record = {
            "id": 137,
            "code": "Тест 04/1",
            "date": "2020-07-17T11:53:08.000",
            "parent_id": 135,
            "demand_id": 85,
            "demand__code": "Тест 04",
            "demand__date": "2020-06-30T00:00:00.000",
            "item_id": 3542260,
            "parent_item_id": 3542258,
            "item__STMP_1_id": 1260666,
            "item__STMP_1__value_str": "Кузов- фургон",
            "item__STMP_2_id": 1978836,
            "item__STMP_2__value_str": "ТЕСТ 2",
            "status_id": 4,
            "status__code": "in_production",
            "status__name": "Сформированы заказы на производство",
            "qty": "2",
            "qty_made": 0,
            "value_sum": "418.0000",
            "isFolder": False,
            "priority": 0,
            "editing": True,
            "deliting": True,
        }
        Launches_viewManager.refreshRows(ids=record.get('id'))



# Generated by Django 3.0.3 on 2020-02-11 04:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0048_auto_20200211_0445'),
    ]

    operations = [
        migrations.AlterField(
            model_name='progresses',
            name='message',
            field=models.TextField(),
        ),
    ]



# Generated by Django 3.1.7 on 2021-03-10 04:52

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0080_auto_20210310_0443'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='id',
            field=models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор'),
        ),
    ]



from isc_common.auth.models.user_e_mails import User_e_mails, User_e_mailsManager
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse


@JsonResponseWithException()
def User_e_mails_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=User_e_mails.objects.
                select_related().
                get_range_rows1(
                request=request,
                function=User_e_mailsManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_e_mails_Add(request):
    return JsonResponse(DSResponseAdd(data=User_e_mails.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_e_mails_Update(request):
    return JsonResponse(DSResponseUpdate(data=User_e_mails.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_e_mails_Remove(request):
    return JsonResponse(DSResponse(request=request, data=User_e_mails.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_e_mails_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=User_e_mails.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_e_mails_Info(request):
    return JsonResponse(DSResponse(request=request, data=User_e_mails.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_e_mails_Copy(request):
    return JsonResponse(DSResponse(request=request, data=User_e_mails.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



from django.urls import path

from clndr.views import calendar_shifts_days

urlpatterns = [

    path('Calendar_shifts_days/Fetch/', calendar_shifts_days.Calendar_shifts_days_Fetch),
    path('Calendar_shifts_days/Add', calendar_shifts_days.Calendar_shifts_days_Add),
    path('Calendar_shifts_days/Update', calendar_shifts_days.Calendar_shifts_days_Update),
    path('Calendar_shifts_days/Remove', calendar_shifts_days.Calendar_shifts_days_Remove),
    path('Calendar_shifts_days/Lookup/', calendar_shifts_days.Calendar_shifts_days_Lookup),
    path('Calendar_shifts_days/Info/', calendar_shifts_days.Calendar_shifts_days_Info),
    path('Calendar_shifts_days/Copy', calendar_shifts_days.Calendar_shifts_days_Copy),

]



# Generated by Django 2.2.4 on 2019-08-22 22:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0032_auto_20190822_2239'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='calendar_shifts_days',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='calendars',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='day',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='day_types',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='holidays',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='shift_day',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='shifts',
            name='id_old',
        ),
    ]



from django.urls import path

from isc_common.auth.views import user_phones

urlpatterns = [

    path('User_phones/Fetch/', user_phones.User_phones_Fetch),
    path('User_phones/Add', user_phones.User_phones_Add),
    path('User_phones/Update', user_phones.User_phones_Update),
    path('User_phones/Remove', user_phones.User_phones_Remove),
    path('User_phones/Lookup/', user_phones.User_phones_Lookup),
    path('User_phones/Info/', user_phones.User_phones_Info),
    path('User_phones/Copy', user_phones.User_phones_Copy),

]



# Generated by Django 3.1.7 on 2021-04-06 10:36

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0092_auto_20210406_1032'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='users_images',
            unique_together={('image', 'main_model', 'props')},
        ),
    ]



# Generated by Django 3.1.7 on 2021-03-31 10:20

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        # ('competitions', '0235_auto_20210331_1020'),
        ('isc_common', '0090_auto_20210326_0949'),
    ]

    operations = [
        migrations.DeleteModel(
            name='AuditModelEx',
        ),
    ]



# from datetime import datetime, time, timedelta
#
#
# class dateTime():
#
#     def __init__(self, value):
#         if isinstance(value, datetime):
#             self.value = datetime(
#                 year=value.year,
#                 month=value.month,
#                 day=value.day,
#                 hour=value.hour,
#                 minute=value.minute,
#                 second=value.second,
#                 microsecond=value.microsecond,
#                 tzinfo=value.tzinfo,
#                 fold=value.fold
#             )
#         elif isinstance(value, time):
#             self.value = time(
#                 fold=value.fold,
#                 hour=value.hour,
#                 minute=value.minute,
#                 second=value.second,
#                 microsecond=value.microsecond,
#             )
#
#     def __add__(self, other):
#         if isinstance(other, dateTime):
#             if isinstance(other.value, time):
#                 return self.value + timedelta(
#                     hours=other.value.hour,
#                     minutes=other.value.minute,
#                     seconds=other.value.second,
#                     microseconds=other.value.microsecond,
#                 )
#             elif isinstance(other.value, datetime):
#                 return self.value + other.value
#         elif isinstance(other, time):
#             if isinstance(other.value, time):
#                 return self.value + timedelta(
#                     hours=other.value.hour,
#                     minutes=other.value.minute,
#                     seconds=other.value.second,
#                     microseconds=other.value.microsecond,
#                 )
#             elif isinstance(other.value, datetime):
#                 return self.value + other.value
#
#     def __str__(self):
#         return str(self.value)



# Generated by Django 3.2.3 on 2021-05-17 05:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0100_alter_images_attfile'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='image_types',
            name='sizes',
        ),
        migrations.AddField(
            model_name='image_types',
            name='height',
            field=models.SmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='image_types',
            name='width',
            field=models.SmallIntegerField(blank=True, null=True),
        ),
    ]



from django.urls import path

from isc_common.views import phones

urlpatterns = [

    path('Phones/Fetch/', phones.Phones_Fetch),
    path('Phones/Add', phones.Phones_Add),
    path('Phones/Update', phones.Phones_Update),
    path('Phones/Remove', phones.Phones_Remove),
    path('Phones/Lookup/', phones.Phones_Lookup),
    path('Phones/Info/', phones.Phones_Info),
    path('Phones/Copy', phones.Phones_Copy),

]



# Generated by Django 3.1.7 on 2021-03-09 07:32

import django.contrib.postgres.fields
import django.utils.timezone
from django.db import migrations, models

import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.files
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0076_auto_20210304_1435'),
    ]

    operations = [
        migrations.CreateModel(
            name='Image_types',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='isc_common.image_types')),
            ],
            options={
                'verbose_name': 'Image types',
            },
        ),
        migrations.CreateModel(
            name='Links',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('link', models.URLField(db_index=True, unique=True)),
            ],
            options={
                'verbose_name': 'Якоря',
            },
        ),
        migrations.CreateModel(
            name='Text_informations',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('text', models.TextField(db_index=True)),
            ],
            options={
                'verbose_name': 'Якоря',
            },
        ),
        migrations.CreateModel(
            name='Images',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('format', isc_common.fields.name_field.NameField(blank=True, null=True, verbose_name='Формат файла')),
                ('mime_type', isc_common.fields.name_field.NameField(blank=True, null=True, verbose_name='MIME тип файла файла')),
                ('size', models.BigIntegerField(default=0, verbose_name='Размер фала')),
                ('real_name', models.TextField(blank=True, db_index=True, null=True, verbose_name='Первоначальное имя файла')),
                ('key', models.BinaryField(blank=True, max_length=200, null=True)),
                ('attfile', isc_common.fields.files.FileFieldEx(blank=True, max_length=255, null=True, upload_to='', verbose_name='Файл')),
                ('file_store', models.CharField(blank=True, max_length=255, null=True, verbose_name='Каталог хранения файла')),
                ('sizes', django.contrib.postgres.fields.ArrayField(base_field=models.SmallIntegerField(), blank=True, null=True, size=None)),
                ('image_type', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='isc_common.image_types')),
            ],
            options={
                'verbose_name': 'Разные картинки ',
            },
        ),
    ]



# Generated by Django 3.1.6 on 2021-02-15 06:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0059_auto_20210121_0639'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='birtdate',
            field=models.DateField(blank=True, null=True),
        ),
    ]



import logging
import os
import socket
import traceback

import paramiko

logger = logging.getLogger(__name__)


class SSH_Client:
    def __init__(self, hostname, username, password, port=22):

        hostkey = None
        try:
            host_keys = paramiko.util.load_host_keys(
                os.path.expanduser("~/.ssh/known_hosts")
            )
        except IOError:
            try:
                # try ~/ssh/ too, because windows can't have a folder named ~/.ssh/
                host_keys = paramiko.util.load_host_keys(
                    os.path.expanduser("~/ssh/known_hosts")
                )
            except IOError:
                logger.debug("*** Unable to open host keys file")
                host_keys = {}

        if hostname in host_keys:
            hostkeytype = host_keys[hostname].keys()[0]
            hostkey = host_keys[hostname][hostkeytype]
            logger.debug("Using host key of type %s" % hostkeytype)

        try:
            t = paramiko.Transport((hostname, port))
            t.connect(
                hostkey,
                username,
                password,
                gss_host=socket.getfqdn(hostname),
                gss_auth=False,
                gss_kex=False,
            )
            self.sftp = paramiko.SFTPClient.from_transport(t)
        except Exception as e:
            logger.error("*** Caught exception: %s: %s" % (e.__class__, e))
            traceback.print_exc()
            try:
                self.t.close()
            except:
                pass

    def get(self, remotepath, localpath, callback=None):
        self.sftp.get(remotepath=remotepath, localpath=localpath, callback=callback)

    def put(self, localpath, remotepath, callback=None, confirm=True):
        self.sftp.put(localpath=localpath, remotepath=remotepath, callback=callback, confirm=confirm)



import logging

from django.forms import model_to_dict
from isc_common import setAttr, delAttr
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager

logger = logging.getLogger(__name__)


class CommonTreeGridManager(AuditManager):

    def createFromRequest(self, request, printRequest=False, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        res = model_to_dict(super().create(**data))
        setAttr(data, 'isFolder', False)
        res.update(data)
        return res

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        delAttr(_data, 'isFolder')
        super().filter(id=request.get_id()).update(**_data)
        return data



from django.urls import path

from tracker.views import message_user_2_user

urlpatterns = [

    path('Message_user_2_user/Fetch/', message_user_2_user.Message_user_2_user_Fetch),
    path('Message_user_2_user/Add', message_user_2_user.Message_user_2_user_Add),
    path('Message_user_2_user/Update', message_user_2_user.Message_user_2_user_Update),
    path('Message_user_2_user/Remove', message_user_2_user.Message_user_2_user_Remove),
    path('Message_user_2_user/Lookup/', message_user_2_user.Message_user_2_user_Lookup),
    path('Message_user_2_user/Info/', message_user_2_user.Message_user_2_user_Info),
    path('Message_user_2_user/Copy', message_user_2_user.Message_user_2_user_Copy),

]



# Generated by Django 2.2.6 on 2019-10-19 17:14

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0073_messagesthemeusersaccess'),
    ]

    operations = [
        migrations.AddField(
            model_name='messagesthemeusersaccess',
            name='props',
            field=bitfield.models.BitField((('all_content', 'Доступ ко всем делам данной темы'),), db_index=True, default=0),
        ),
    ]



import logging

from django.db.models import TextField

from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class Model_text_informationsQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        text = defaults.get('text')

        if text != '' and text is not None:
            try:
                item = super().get(**kwargs)
                text_item, _ = Text_informations.objects.update_or_create(id=item.id, defaults=dict(text=text))
            except self.model.DoesNotExist:
                text_item = Text_informations.objects.create(text=text)
            defaults = dict(text=text_item)
            return super().update_or_create(defaults=defaults, **kwargs)
        else:
            return super().delete()


class Text_informationsQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Text_informationsManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Text_informationsQuerySet(self.model, using=self._db)


class Text_informations(AuditModel):
    text = TextField(db_index=True)

    objects = Text_informationsManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Якоря'



from django.urls import path

from reports.views import jasper_reports_users

urlpatterns = [

    path('Jasper_reports_users/Fetch/', jasper_reports_users.Jasper_reports_users_Fetch),
    path('Jasper_reports_users/Add', jasper_reports_users.Jasper_reports_users_Add),
    path('Jasper_reports_users/Update', jasper_reports_users.Jasper_reports_users_Update),
    path('Jasper_reports_users/Remove', jasper_reports_users.Jasper_reports_users_Remove),
    path('Jasper_reports_users/Lookup/', jasper_reports_users.Jasper_reports_users_Lookup),
    path('Jasper_reports_users/Info/', jasper_reports_users.Jasper_reports_users_Info),
    path('Jasper_reports_users/Copy', jasper_reports_users.Jasper_reports_users_Copy),

]



# Generated by Django 2.2.2 on 2019-06-10 15:30

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0004_auto_20190526_1042'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documents_param_1c',
            name='document',
            field=isc_common.fields.related.ForeignKeyCascade(default=None, on_delete=django.db.models.deletion.CASCADE, to='one_c.Document_1c'),
        ),
        migrations.CreateModel(
            name='Documents_param_cross_1c',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('document', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='one_c.Document_1c')),
                ('param', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='one_c.Documents_param_1c')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-21 06:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0058_auto_20210121_0633'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='debug',
            name='deleted_at',
        ),
        migrations.RemoveField(
            model_name='debug',
            name='deliting',
        ),
        migrations.RemoveField(
            model_name='debug',
            name='editing',
        ),
        migrations.RemoveField(
            model_name='debug',
            name='lastmodified',
        ),
        migrations.AddField(
            model_name='debug',
            name='params',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='debug',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



import logging
import os

from django.conf import settings

from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class Model_imagesQuerySet(AuditQuerySet):
    pass


class Model_imagesManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
        }
        return res

    def get_queryset(self):
        return Model_imagesQuerySet(self.model, using=self._db)


class ImageNotFound(Exception):
    pass


class Model_images(AuditModel):
    @classmethod
    def get_image(cls, file_name, path=None):
        if file_name is None:
            return None

        if os.altsep is not None:
            have_path = file_name.find(os.altsep) != -1
        else:
            have_path = file_name.find(os.sep) != -1

        if have_path:
            if os.altsep is None:
                file_name1 = f'{settings.OLD_SITE_BASE_DIR}{file_name}'.replace(os.sep, '\\')
            else:
                file_name1 = f'{settings.OLD_SITE_BASE_DIR}{file_name}'.replace(os.altsep, os.sep)
            try:
                return cls.objects.get(path__upper=file_name1.upper()).path
            except cls.DoesNotExist:
                return None

        query = cls.objects.filter(file_name=file_name).order_by('-date')
        if query.count() == 0:
            return None
        else:
            _list = list(map(lambda x: x.path, query))
            if path is None:
                return _list[0]

            for item in _list:
                if item.find(path) != -1:
                    return item
            return None

    @classmethod
    def update_or_create_image(cls, main_model, code, path, file_name=None, self_image_field=False, name=None, exception=True, full_name_image=None, image_field_name=None, pk_name='id', defaults=None, update_from_sftp=False):
        from isc_common.models.image_types import Image_types
        from isc_common.models.images import Images
        from lfl_admin.common.models.site_lfl_images import Site_lfl_images

        if name is None:
            name = code

        if file_name is not None and file_name.strip() == '':
            file_name = None

        if full_name_image is None:
            full_name_image = Site_lfl_images.get_image(file_name=file_name, path=path)

        if full_name_image is not None:

            image_type, created = Image_types.objects.get_or_create(code=code, defaults=dict(name=name))
            image, created = Images.objects.update_or_create(image_type=image_type, real_name=full_name_image)

            if os.path.exists(full_name_image):
                image1, created = Images.create_update(id=image.id, image_type=image_type, real_file_name=full_name_image)

            elif update_from_sftp is True:
                full_name_image_replaced = full_name_image.replace(settings.PATH_IMAGES_REPLACE.get('old_path'), settings.PATH_IMAGES_REPLACE.get('new_path'))
                full_name_image_replaced = full_name_image_replaced.replace(os.altsep if os.altsep is not None else '\\', os.sep)
                file_name = os.path.split(full_name_image_replaced)[1]
                localpath = f"{settings.PATH_IMAGES_REPLACE.get('tmp_path')}{file_name}"
                settings.SSH_CLIENT.get(remotepath=full_name_image_replaced, localpath=localpath)

                if os.path.exists(localpath):
                    image1, created = Images.create_update(
                        id=image.id,
                        image_type=image_type,
                        real_file_name=full_name_image,
                        tmp_file_name=localpath
                    )

            if self_image_field is False:
                return cls.objects.update_or_create(image=image, main_model=main_model, defaults=defaults)
            else:
                eval(f'main_model._meta.concrete_model.objects.filter({pk_name}=main_model.{pk_name}).update({image_field_name}=image)', dict(), dict(image=image, main_model=main_model))

            logger.debug(f'{file_name} : found')

        elif file_name is not None:
            if exception is True:
                raise ImageNotFound(f'{file_name} : not found')
            else:
                logger.debug(f'{file_name} : not found !!!')
        return None, None

    objects = Model_imagesManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Базовый класс'
        abstract = True



import logging

from crypto.models.crypto_file import Crypto_file, CryptoManager, CryptoQuerySet
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.image_types import Image_types

logger = logging.getLogger(__name__)


class ImagesQuerySet(CryptoQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        if kwargs.get('real_name') is not None:
            try:
                return super().get(real_name=kwargs.get('real_name'))
            except self.model.DoesNotExist:
                return super().create(**kwargs)
        else:
            return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class ImagesManager(CryptoManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return ImagesQuerySet(self.model, using=self._db)


class Images(Crypto_file):
    image_type = ForeignKeyProtect(Image_types)
    objects = ImagesManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Разные картинки '
